

/*
 * Author: Tomi Jylhä-Ollila, Finland 2010
 *
 * This file is part of Kunquat.
 *
 * CC0 1.0 Universal, http://creativecommons.org/publicdomain/zero/1.0/
 *
 * To the extent possible under law, Kunquat Affirmers have waived all
 * copyright and related or neighboring rights to Kunquat.
 */


#ifndef K_SAMPLE_PARAMS_H
#define K_SAMPLE_PARAMS_H


#include <stdint.h>

#include <File_base.h>


typedef enum
{
    /// Uninitialised.
    SAMPLE_FORMAT_NONE = 0,
    /// WavPack.
    SAMPLE_FORMAT_WAVPACK,
    /// Vorbis.
    // SAMPLE_FORMAT_VORBIS,
    /// Sentinel -- not a valid format.
    SAMPLE_FORMAT_LAST
} Sample_format;


typedef enum
{
    SAMPLE_LOOP_OFF = 0, ///< No loop.
    SAMPLE_LOOP_UNI,     ///< Unidirectional (forward) loop.
    SAMPLE_LOOP_BI       ///< Bidirectional loop.
} Sample_loop;


/**
 * Common parameters for a Sample.
 */
typedef struct Sample_params
{
    Sample_format format; ///< The file format.
    double mid_freq;      ///< The playback frequency used to represent 440 Hz tone.
    Sample_loop loop;     ///< Loop setting.
    uint64_t loop_start;  ///< Loop start.
    uint64_t loop_end;    ///< Loop end (the frame at this index will not be played).
} Sample_params;


/**
 * Initialises Sample parameters.
 *
 * \param params   The Sample parameters -- must not be \c NULL.
 *
 * \return   The parameter \a params.
 */
Sample_params* Sample_params_init(Sample_params* params);


/**
 * Parses the Sample parameters from a string.
 *
 * \param params   The Sample parameters -- must not be \c NULL.
 * \param str      The textual description.
 * \param state    The Read state -- must not be \c NULL.
 *
 * \return   \c true if successful, otherwise \c false.
 */
bool Sample_params_parse(Sample_params* params,
                         char* str,
                         Read_state* state);


/**
 * Copies Sample parameters.
 *
 * \param dest   The copy destination -- must not be \c NULL.
 * \param src    The copy source -- must not be \c NULL.
 *
 * \return   The parameter \a dest.
 */
Sample_params* Sample_params_copy(Sample_params* dest, Sample_params* src);


#endif // K_SAMPLE_PARAMS_H


